

package roombacomm;

import java.io.*;

/**
 *  Spy on the Roomba as it goes about its normal business

 * <p>
 *  Run it with something like: <pre>
 *   java roombacomm.Tribble /dev/cu.KeySerial1
 *  </pre>
 *
 */
public class Spiral {
    
    String usage = 
        "Usage: \n"+
        "  roombacomm.Spiral <serialportname> [options]\n" +
        "where [options] can be one or more of:\n"+
        " -pause <n>   -- pause n millseconds between sensor read\n"+
        " -debug       -- turn on debug output\n"+
        " -hwhandshake -- use hardware-handshaking, for Windows Bluetooth\n"+
        "\n";

    boolean debug = false;
    boolean hwhandshake = false;
    int pausetime = 500;
    int speed = 500;
    int radius = 10;
    int radius_inc = 20;
    int radius_max = 1000;

    public static void main(String[] args) {
        new Spiral(args);
    }
    
    public Spiral(String[] args) {
        if( args.length == 0 ) {
            System.out.println( usage );
            System.exit(0);
        }
        String portname = args[0];  // e.g. "/dev/cu.KeySerial1"
        
        for( int i=0; i<args.length; i++ ) {
            if( args[i].endsWith("debug") ) 
                debug = true;
            else if( args[i].endsWith("hwhandshake") )
                hwhandshake = true;
            else if( args[i].endsWith("pause") ) {
                i++;
                int p = 0;
                try { p = Integer.parseInt( args[i] ); }
                catch( NumberFormatException nfe ) { }
                if( p!=0 ) pausetime = p;
            }
        }
        
        RoombaCommSerial roombacomm = new RoombaCommSerial();
        roombacomm.debug = debug;
        roombacomm.waitForDSR = hwhandshake;
        
        if( ! roombacomm.connect( portname ) ) {
            System.out.println("Couldn't connect to "+portname);
            System.exit(1);
        }
        
        System.out.println("Roomba startup");
        roombacomm.startup();
        roombacomm.control();
        
        System.out.println("Press return to exit.");
        boolean running = true;
        while( running ) { 
            System.out.println("radius = "+radius);

            roombacomm.drive( speed, radius );
            radius += radius_inc;
            if( Math.abs(radius) == 410 ) {
                speed = speed;
                radius_inc = -radius_inc;
            }

            try { 
                if( System.in.available() != 0 ) {
                    System.out.println("key pressed");
                    running = false;
                }
            } catch( IOException ioe ) { }
            
            roombacomm.pause( pausetime );
        }
        
    }
    
}
